import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt

def expFit(x, A, b, c):
	return A * np.exp(-x / b) + c

SMALL = 8
MEDIUM = 14
LARGE = 12

fig = plt.figure(figsize=(3.2,3.2))

kbt = 4.11

# Read data
dFname = "../results/SinglePolymer_PersitenceLengths.dat"

data = np.genfromtxt(dFname, skip_header=1, delimiter=",").transpose()

# Plot
plt.errorbar(data[1], data[4], yerr=data[5], fmt="xC%d" % (0))

# Try a fit
fit = expFit
guess = [1.0,0.1,0.0]

popt2, pcov2 = curve_fit(fit, data[1], data[4], guess)
cData = np.linspace(data[1][0], data[1][-1],50)
plt.plot(cData, fit(cData, *popt2), "--C%d" % (1))

# Prettify
plt.xlim(-0.1,plt.xlim()[1])
plt.ylim(-0.1,plt.ylim()[1])
plt.xlabel(r"$\Delta l / 2R$", fontsize=MEDIUM)
plt.ylabel(r"L'$_p$", fontsize=MEDIUM)

#plt.legend(loc=0, fontsize=MEDIUM)
ax00 = plt.gca()
ax00.set_xticklabels(["%.1f" % bit for bit in ax00.get_xticks()], {"fontsize":MEDIUM})
ax00.set_yticklabels(["%.1f" % bit for bit in ax00.get_yticks()], {"fontsize":MEDIUM})
plt.tight_layout()

# Save
oFname = "../results/SinglePolymer_PersitenceLengthsNorm.png"
plt.savefig(oFname, dpi=300)
plt.clf()

# Plot
plt.errorbar(data[1], data[2], yerr=data[3], fmt="xC%d" % (0))

# Try a fit
fit = expFit
guess = [1.0,0.1,0.0]

popt, pcov = curve_fit(fit, data[1], data[2], guess)
cData = np.linspace(data[1][0], data[1][-1],50)
plt.plot(cData, fit(cData, *popt), "--C%d" % (1))

# Now, determine the values of dL we'll need to do subsequent sims. Do this via trial and error guessing at the value
R = 2.5
N = 5
LpOnLcs = [0.5, 1.0, 1.5]

iIndex = 0
for LpOnLc in LpOnLcs:

	xUp = 10.0
	xLow = 0.0
	err = 1.0
	while (np.fabs(err) > 0.01):
		x = 0.5 * (xUp + xLow)

		LpOnLcTest = expFit(x, *popt)
		LpOnLcTest /= 2 * R * ( (x + 1) * (N - 1) + 1)
		
		err = (LpOnLcTest - LpOnLc) / LpOnLc
		if(err > 0):
			xLow = x
		else:
			xUp = x
	

	# Add to graph	
	Lp = expFit(x, *popt)
	Lc = Lp / LpOnLc
	print("Lp = %.3f, Lc = %.3f, dL / 2 * R = %.3f, k = %.3f" % (Lp, Lc, x, kbt / np.power(x * 2 * R, 2)))

	# Plot these on the graph
	axMin = [-0.1, -0.1]
	plt.plot([axMin[0],x], [Lp,Lp], "--C%d" % (2 + iIndex), label="Lp/Lc = %.1f" % (LpOnLc))
	plt.plot([x,x], [axMin[1],Lp], "--C%d" % (2 + iIndex))
	
	iIndex += 1
		
# Prettify
plt.xlim(-0.1,plt.xlim()[1])
plt.ylim(-0.1,plt.ylim()[1])
plt.xlabel(r"$\Delta l / 2R$", fontsize=MEDIUM)
plt.ylabel(r"L$_p$ (nm)", fontsize=MEDIUM)

plt.legend(loc=0, fontsize=SMALL)

ax00 = plt.gca()
ax00.set_xticklabels(["%.1f" % bit for bit in ax00.get_xticks()], {"fontsize":MEDIUM})
ax00.set_yticklabels(["%.1f" % bit for bit in ax00.get_yticks()], {"fontsize":MEDIUM})

plt.tight_layout()

# Save
oFname = "../results/SinglePolymer_PersitenceLengths.png"
plt.savefig(oFname, dpi=300)

